//						Simple 3D Scene							   \\
// Simple scene demonstrating Planar shadows, Stencil reflections, \\
// use of shaders (Vertex and Pixel shader),.X model loading and   \\
// various transformations with DirectX  9.0c                      \\
// MY CHYE - Coursework 1 CS09020								   \\
// 30th January 2009                                               \\
//////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

#include "d3dUtility.h"
#include "camera.h"
#include <atlbase.h>
#include <math.h>

//
// Globals
//
#define LIGHTDIRECTION D3DXVECTOR4 (0, -0.870, -0.8, 0.0f); // Light Direction
#define kMoveAmt (6.0) // How fast the camera moves
#define DEG2RAD(x) (x * (D3DX_PI / 180.0f)) // Converts degrees to radians
#define RAD2DEG(x) (x * (180.0f / D3DX_PI)) // Converts radians to degrees
#define SCALE (3) // Scale for floor, wall and mirror
#define Width (800.0f)
#define Height (600.0f)
#define D3DFVF_CUSTOMVERTEX  D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1
#define PI (3.142)

IDirect3DDevice9*     Device = 0; 

// Mesh global variables
LPD3DXMESH myMesh;
DWORD numMaterials;
D3DMATERIAL9* material;
LPDIRECT3DTEXTURE9* texture;   
IDirect3DTexture9* mWhiteTex; 

// Textures for Scene
IDirect3DTexture9* FloorTex  = 0;
IDirect3DTexture9* WallTex  = 0;
IDirect3DTexture9* MirrorTex  = 0;

//viewing matrices
D3DXMATRIX mwvp;
D3DXMATRIX mproj;
D3DXMATRIX mView;
D3DXMATRIX mWorld;
D3DXMATRIX mWorldView;
D3DXMATRIX mViewIT;
D3DXMATRIX mWorldIT;
D3DXMATRIX mWorldViewIT;

D3DXCOLOR mcd,mca,mcs,lc;

D3DXVECTOR4 vLightDirs,vLightCols;
D3DXVECTOR3 CarPos(0, 2.0f, -15.0f);

IDirect3DVertexBuffer9* VB = 0;
ID3DXEffect* mFX = 0; 
ID3DXBuffer* errors = 0;

//Handles
D3DXHANDLE mhTech;
D3DXHANDLE mhDiffuseMtrl;
D3DXHANDLE mhAmbMtrl;
D3DXHANDLE mhSpecMtrl;
D3DXHANDLE mhLightCol;
D3DXHANDLE mhvLightDirs;
D3DXHANDLE mhvLightCols;
D3DXHANDLE mhgTex;

//Viewing matrices used for shaders
D3DXHANDLE mhWVP;
D3DXHANDLE mhWorldView;
D3DXHANDLE mhWorldViewIT;
D3DXHANDLE mhProj;

float rotY = 0;

// Our Camera
Camera TheCamera(Camera::LANDOBJECT);

//
// Classes and Structures
//

struct Vertex
{
	Vertex(){}
	Vertex(float x, float y, float z, 
		   float nx, float ny, float nz,
		   float u, float v)
	{
		_x  = x;  _y  = y;  _z  = z;
		_nx = nx; _ny = ny; _nz = nz;
		_u  = u;  _v  = v;
	}
	float _x, _y, _z;
	float _nx, _ny, _nz;
	float _u, _v;

	static IDirect3DVertexDeclaration9* Decl;	
	static const DWORD FVF;
};
const DWORD Vertex::FVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1;



///////////////////////////////////////////// WORLD RENDERING FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
// Contains functions used to create the whole world																		\
///////////////////////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void RenderWorld()
{	
	// Draw Car first
	D3DXMATRIX matTrans, matScale, matRotate;
	D3DXMatrixRotationY(&matRotate, rotY); 
	D3DXMatrixTranslation(&matTrans, CarPos.x, CarPos.y, CarPos.z);
	//D3DXMatrixScaling(&matScale, 1.2f, 1.2f, 1.2f);
	mWorld = matRotate*matTrans;
	mWorldView = mWorld*mView;
	mFX->SetMatrix(mhWorldView,&(mWorldView));
	//determine the inversetranspose matrices of worldview and view matrices
	D3DXMatrixInverse(&mWorldViewIT,NULL, &mWorldView);
	D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewIT);
	mFX->SetMatrix(mhWorldViewIT,&(mWorldViewIT));
	mFX->SetMatrix(mhWVP,&(mWorld*mView*mproj));
	mFX->SetVector( mhvLightDirs, (D3DXVECTOR4*)&vLightDirs);
	mFX->SetVector( mhvLightCols, (D3DXVECTOR4*)&vLightCols);
	//Begin passes
	UINT numPasses=0;
	mFX->Begin(&numPasses,0);
	mFX->BeginPass(0);	
		
		for(DWORD t = 0; t < numMaterials; t++)    // loop through each subset
		{
			
			mcd = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
			mcs = D3DXCOLOR(1.0f,1.0f, 1.0f, 1.0f);
			mca = D3DXCOLOR(0.3f, 0.3f,0.3f, 1.0f);
			
			mFX->SetValue(mhDiffuseMtrl,&(mcd) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhAmbMtrl,&(mca) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));

			//Device->SetMaterial(&material[i]);    // set the appropriate material for the subset
			Device->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
			mFX->SetValue(mhDiffuseMtrl,&(material[t].Diffuse) ,sizeof(D3DCOLORVALUE));
			mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhAmbMtrl,&(material[t].Ambient) ,sizeof(D3DCOLORVALUE));
			if(texture[t] != NULL)    
			{
				mFX->SetTexture(mhgTex, texture[t]);    
			}
			else
			{
				mFX->SetTexture(mhgTex,mWhiteTex);    
			}
			mFX->CommitChanges();
		
			myMesh->DrawSubset(t);    
		}
	
		// Now draw the rest of the scene
		D3DXMATRIX mTemps;
		D3DXMatrixIdentity(&mTemps);
		mWorld = mTemps;
		mWorldView = mWorld*mView;
		mFX->SetMatrix(mhWorldView,&(mWorldView));
		//determine the inversetranspose matrices of worldview and view matrices
		D3DXMatrixInverse(&mWorldViewIT,NULL, &mWorldView);
		D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewIT);
		mFX->SetMatrix(mhWorldViewIT,&(mWorldViewIT));
		mFX->SetMatrix(mhWVP,&(mWorld*mView*mproj)); 
		//update colour of ground
		mcd = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
		mcs = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
		mca = D3DXCOLOR(0.3f, 0.3f, 0.3f, 1.0f);
		mFX->SetValue(mhDiffuseMtrl,&(mcd) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhAmbMtrl,&(mca) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));
		  
		//UINT numPasses=0;
		mFX->Begin(&numPasses,0);

		for(int i=0; i < numPasses; ++i)
		{
			mFX->BeginPass(i);
			mFX->SetTexture(mhgTex, FloorTex);
			mFX->CommitChanges();			
			Device->SetStreamSource(0, VB, 0, sizeof(Vertex));
			Device->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);
			mFX->SetTexture(mhgTex, WallTex);
			mFX->CommitChanges();
			Device->DrawPrimitive(D3DPT_TRIANGLELIST, 6, 4);	
			mFX->SetTexture(mhgTex, MirrorTex);
			mFX->CommitChanges();
			Device->DrawPrimitive(D3DPT_TRIANGLELIST, 18, 2);	
			mFX->EndPass();	
		}
		mFX->End();
}

// Function to create Mirror effect
void RenderMirror()
{
	Device->SetRenderState(D3DRS_STENCILENABLE,    true);
    Device->SetRenderState(D3DRS_STENCILFUNC,      D3DCMP_ALWAYS);
    Device->SetRenderState(D3DRS_STENCILREF,       0x1);
    Device->SetRenderState(D3DRS_STENCILMASK,      0xffffffff);
    Device->SetRenderState(D3DRS_STENCILWRITEMASK, 0xffffffff);
    Device->SetRenderState(D3DRS_STENCILZFAIL,     D3DSTENCILOP_KEEP);
    Device->SetRenderState(D3DRS_STENCILFAIL,      D3DSTENCILOP_KEEP);
    Device->SetRenderState(D3DRS_STENCILPASS,      D3DSTENCILOP_REPLACE);

	// disable writes to the depth and back buffers
    Device->SetRenderState(D3DRS_ZWRITEENABLE, false);
    Device->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
    Device->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ZERO);
    Device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

    	D3DXMATRIX mTemps;
		D3DXMatrixIdentity(&mTemps);
		mWorld = mTemps;
		mWorldView = mWorld*mView;
		mFX->SetMatrix(mhWorldView,&(mWorldView));
		//determine the inversetranspose matrices of worldview and view matrices
		D3DXMatrixInverse(&mWorldViewIT,NULL, &mWorldView);
		D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewIT);
		mFX->SetMatrix(mhWorldViewIT,&(mWorldViewIT));
		mFX->SetMatrix(mhWVP,&(mWorld*mView*mproj)); 
		
		mcd = D3DXCOLOR(0.0f, 0.0f, 0.0f, 0.0f);
		mcs = D3DXCOLOR(1.0f, 1.0f, 1.0f, 0.0f);
		mca = D3DXCOLOR(0.3f, 0.3f, 0.3f, 0.5f);
		mFX->SetValue(mhDiffuseMtrl,&(mcd) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhAmbMtrl,&(mca) ,sizeof(D3DXCOLOR));
		mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));
		  
		UINT numPasses=0;
		mFX->Begin(&numPasses,0);

		for(int i=0; i < numPasses; ++i)
		{
			mFX->BeginPass(i);					
			Device->SetStreamSource(0, VB, 0, sizeof(Vertex));				
			mFX->SetTexture(mhgTex, MirrorTex);
			mFX->CommitChanges();
			Device->DrawPrimitive(D3DPT_TRIANGLELIST, 18, 2);	
			mFX->EndPass();	
		}
		mFX->End();

	// re-enable depth writes
	Device->SetRenderState( D3DRS_ZWRITEENABLE, true );

	// Only draw to the mirror
	Device->SetRenderState(D3DRS_STENCILFUNC,  D3DCMP_EQUAL);
    Device->SetRenderState(D3DRS_STENCILPASS,  D3DSTENCILOP_KEEP);

	// Position reflection
	D3DXMATRIX W, T, R;
	D3DXPLANE plane(0.0f, 0.0f, 1.0f, 0.0f); // xy plane
	D3DXMatrixReflect(&R, &plane);

	D3DXMatrixTranslation(&T,
		CarPos.x, 
		CarPos.y,
		CarPos.z); 

	W = T * R;

	// clear depth buffer and blend the reflected teapot with the mirror
	Device->Clear(0, 0, D3DCLEAR_ZBUFFER, 0, 1.0f, 0);
	Device->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_DESTCOLOR);
    Device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);	
	
	D3DXMATRIX matTrans, matScale, matRotate;
	D3DXMatrixRotationY(&matRotate, rotY);
	mWorld = matRotate*W;
	mWorldView = mWorld*mView;
	mFX->SetMatrix(mhWorldView,&(mWorldView));
	//determine the inversetranspose matrices of worldview and view matrices
	D3DXMatrixInverse(&mWorldViewIT,NULL, &mWorldView);
	D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewIT);
	mFX->SetMatrix(mhWorldViewIT,&(mWorldViewIT));
	mFX->SetMatrix(mhWVP,&(mWorld*mView*mproj)); 

	// Reverse cull mode
	Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	//UINT numPasses=0;
	mFX->Begin(&numPasses,0);
	mFX->BeginPass(0);	
		
		for(DWORD t = 0; t < numMaterials; t++)    // loop through each subset
		{			
			mcd = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
			mcs = D3DXCOLOR(1.0f,1.0f, 1.0f, 1.0f);
			mca = D3DXCOLOR(0.3f, 0.3f,0.3f, 1.0f);
			
			mFX->SetValue(mhDiffuseMtrl,&(mcd) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhAmbMtrl,&(mca) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));

			Device->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
			mFX->SetValue(mhDiffuseMtrl,&(material[t].Diffuse) ,sizeof(D3DCOLORVALUE));
			mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhAmbMtrl,&(material[t].Ambient) ,sizeof(D3DCOLORVALUE));
			if(texture[t] != NULL)    
			{
				mFX->SetTexture(mhgTex, texture[t]);    
			}
			else
			{
				mFX->SetTexture(mhgTex,mWhiteTex);    
			}
			mFX->CommitChanges();		
			myMesh->DrawSubset(t);    
		}	
	
	// Restore render states.
	Device->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
	Device->SetRenderState( D3DRS_STENCILENABLE, false);
	Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
}

// Function to create shadow
void drawShadow()
{
	Device->SetRenderState(D3DRS_STENCILENABLE,    true);
    Device->SetRenderState(D3DRS_STENCILFUNC,      D3DCMP_EQUAL);
    Device->SetRenderState(D3DRS_STENCILREF,       0x0);
    Device->SetRenderState(D3DRS_STENCILMASK,      0xffffffff);
    Device->SetRenderState(D3DRS_STENCILWRITEMASK, 0xffffffff);
    Device->SetRenderState(D3DRS_STENCILZFAIL,     D3DSTENCILOP_KEEP);
    Device->SetRenderState(D3DRS_STENCILFAIL,      D3DSTENCILOP_KEEP);
    Device->SetRenderState(D3DRS_STENCILPASS,      D3DSTENCILOP_INCR); // increment to 1

	// position shadow	
	D3DXPLANE groundPlane(0.0f, -1.0f, 0.0f, 0.0f);

	D3DXMATRIX S;
	D3DXMatrixShadow(
		&S,
		&vLightDirs,
		&groundPlane);

	D3DXMATRIX T;
	D3DXMatrixTranslation(
		&T,
		CarPos.x,
		CarPos.y,
		CarPos.z);

	//Translate shadow slightly upwards to avoid Z fighting with ground plane
	D3DXMATRIX TEMP;
	D3DXMatrixTranslation(&TEMP, 0, 0.001, 0); 
	D3DXMATRIX W =  T * S ;
	D3DXMATRIX matTrans, matScale, matRotate;
	D3DXMatrixRotationY(&matRotate, rotY);
	mWorld = matRotate*W*TEMP;
	mWorldView = mWorld*mView;
	mFX->SetMatrix(mhWorldView,&(mWorldView));
	//determine the inversetranspose matrices of worldview and view matrices
	D3DXMatrixInverse(&mWorldViewIT,NULL, &mWorldView);
	D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewIT);
	mFX->SetMatrix(mhWorldViewIT,&(mWorldViewIT));
	mFX->SetMatrix(mhWVP,&(mWorld*mView*mproj));		

	// alpha blend the shadow
	Device->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	Device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	Device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	//Begin passes
	UINT numPasses=0;
	mFX->Begin(&numPasses,0);
	mFX->BeginPass(0);
	for(DWORD t = 0; t < numMaterials; t++)   
		{
			mcd = D3DXCOLOR(0.0f, 0.0f, 0.0f, 0.5f);
			mcs = D3DXCOLOR(0.0f,0.0f, 0.0f, 0.5f);
			mca = D3DXCOLOR(0.0f, 0.0f,0.0f, 0.5f);
			mFX->SetValue(mhDiffuseMtrl,&(mcd) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhSpecMtrl,&(mcs) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhAmbMtrl,&(mca) ,sizeof(D3DXCOLOR));
			mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));
			mFX->SetTexture(mhgTex,mWhiteTex);   
			mFX->CommitChanges();
			myMesh->DrawSubset(t);    // draw the subset

		}	
	Device->SetRenderState(D3DRS_ZENABLE, true);
	Device->SetRenderState(D3DRS_ALPHABLENDENABLE, false);
	Device->SetRenderState(D3DRS_STENCILENABLE,    false);	
}

/////////////////////////////////////// END OF WORLD RENDERING FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

//
// Framework Functions
//

void setUpRenderStates()
{	
	//disable z write enable as rendering transparent obf
	Device->SetRenderState(D3DRS_ZWRITEENABLE,true);
}

void CreateFX()
{
	D3DXCreateEffectFromFile(Device, "Phong_Shader.fx", 0, 0, D3DXSHADER_DEBUG,0, &mFX,&errors);
	if(errors)
		MessageBox(0, (char*)errors->GetBufferPointer(),0,0);

	//Obtaining handle to an effect	
	mhTech = mFX ->GetTechniqueByName("PhongShaderTech");
	mhWVP = mFX->GetParameterByName(0, "gWVP"); 
	mhWorldView = mFX->GetParameterByName(0, "gWorldView"); 
	mhWorldViewIT = mFX->GetParameterByName(0, "gWorldViewIT"); 
	mhDiffuseMtrl=mFX->GetParameterByName(0,"gDiffuseMtrl");
	mhSpecMtrl=mFX->GetParameterByName(0,"gSpecMtrl");
	mhAmbMtrl=mFX->GetParameterByName(0,"gAmbMtrl");
	mhvLightCols=mFX->GetParameterByName(0,"gLightCol");
	mhvLightDirs= mFX->GetParameterByName(0,"gLightDir");
	mhgTex=mFX->GetParameterByName(0, "gTex");
}

// Grabs the mouse input to control the pitch/yaw of the camera
void CameraMouseInput()
{
	const float kMaxAngle = 45.0f; // Max degree of rotation
	static float pitchAmt = 0.0f; // Amount we've looked up or down
	
	// Get the middle of the screen
	int midScrX = GetSystemMetrics(SM_CXSCREEN) >> 1;
	int midScrY = GetSystemMetrics(SM_CYSCREEN) >> 1;
	
	float amt;
	POINT pt = {0};
	
	GetCursorPos(&pt); // Get the current mouse position
		
	// Rotate left/right
	amt = float(pt.x - midScrX) * (kMoveAmt/30);
	TheCamera.yaw(DEG2RAD(amt));
	
	// Calculate amount to rotate up/down
	amt = float(midScrY - pt.y) * (kMoveAmt/30);

	// Clamp pitch amount
	if(pitchAmt + amt <= -kMaxAngle)
	{
		amt = -kMaxAngle - pitchAmt;
		pitchAmt = -kMaxAngle;
	}
	else if(pitchAmt + amt >= kMaxAngle)
	{
		amt = kMaxAngle - pitchAmt;
		pitchAmt = kMaxAngle;
	}
	else
	{
		pitchAmt += amt;
	}	
	// Pitch camera
	TheCamera.pitch(-DEG2RAD(amt)); 		
	SetCursorPos(midScrX, midScrY); // Set cursor back to the middle of the screen
}


void SetUpLights()
{
	// Setup our lighting parameters	
	vLightDirs = LIGHTDIRECTION ; //points downward
	vLightCols = D3DXVECTOR4( 1.0f, 1.0f, 1.0f, 1.0f );
}

bool Setup()
{
D3DXCreateTextureFromFile(Device, "whitetex.dds", &mWhiteTex);

	// local variable ()
		LPD3DXBUFFER bufMeshMaterial;

		D3DXLoadMeshFromX("f40b.x",    // load this file
						  D3DXMESH_SYSTEMMEM,    // load the mesh into system memory
						  Device,    // the Direct3D Device
						  NULL,    // we aren't using adjacency
						  &bufMeshMaterial,    // put the materials here
						  NULL,    // we aren't using effect instances
						  &numMaterials,    // the number of materials in this model
						  &myMesh);    // put the mesh here
	
		ID3DXMesh* temp=0;
		myMesh->CloneMeshFVF( 0, D3DFVF_CUSTOMVERTEX, Device, &temp);
			myMesh->Release();
			myMesh = temp;			
			D3DXComputeNormals(myMesh, NULL);       
	
		// retrieve the pointer to the buffer containing the material information
		D3DXMATERIAL* tempMaterials = (D3DXMATERIAL*)bufMeshMaterial->GetBufferPointer();

		// create a new material buffer for each material in the mesh..likewise for texture
		 material = new D3DMATERIAL9[numMaterials];
		 texture = new LPDIRECT3DTEXTURE9[numMaterials];

		for(DWORD i = 0; i < numMaterials; i++)    
		{
			material[i] = tempMaterials[i].MatD3D;    // get the material info...
			material[i].Ambient = material[i].Diffuse;    // and make ambient the same as diffuse
		
		 USES_CONVERSION;    // allows certain string conversions
        
        if(FAILED(D3DXCreateTextureFromFile(Device,
                                            tempMaterials[i].pTextureFilename,
                                            &texture[i])))
        texture[i] = mWhiteTex;    // if there is no texture, set the texture to white
		}		

	//Set up lights
	SetUpLights();

	//Initialise camera			
	D3DXVECTOR3 camPos(0.0f, 7.0f, -55.0f);
	TheCamera.setPosition(&camPos);

	//create effects
	CreateFX();

	//
	// Create vertex and index buffers.
	//
	Device->CreateVertexBuffer(
		24 * sizeof(Vertex), 
		0,
		Vertex::FVF,
		D3DPOOL_MANAGED,
		&VB,
		0);

	Vertex* v = 0;
	VB->Lock(0, 0, (void**)&v, 0);
	
	// floor	
	v[0] = Vertex(-7.5f*SCALE, 0.0f*SCALE, -10.0f*SCALE, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f);
	v[1] = Vertex(-7.5f*SCALE, 0.0f*SCALE,   0.0f*SCALE, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f);
	v[2] = Vertex( 7.5f*SCALE, 0.0f*SCALE,   0.0f*SCALE, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f);
	
	v[3] = Vertex(-7.5f*SCALE, 0.0f*SCALE, -10.0f*SCALE, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f);
	v[4] = Vertex( 7.5f*SCALE, 0.0f*SCALE,   0.0f*SCALE, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f);
	v[5] = Vertex( 7.5f*SCALE, 0.0f*SCALE, -10.0f*SCALE, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f);

	//// wall
	v[6]  = Vertex(-7.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[7]  = Vertex(-7.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f);
	v[8]  = Vertex(-2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	
	v[9]  = Vertex(-7.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[10] = Vertex(-2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	v[11] = Vertex(-2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f);

	v[12] = Vertex(2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[13] = Vertex(2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f);
	v[14] = Vertex(7.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	
	v[15] = Vertex(2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[16] = Vertex(7.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	v[17] = Vertex(7.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f);

	v[18] = Vertex(-2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[19] = Vertex(-2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f);
	v[20] = Vertex( 2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	
	v[21] = Vertex(-2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f);
	v[22] = Vertex( 2.5f*SCALE, 5.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f);
	v[23] = Vertex( 2.5f*SCALE, 0.0f*SCALE, 0.0f*SCALE, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f);

	VB->Unlock();

	 if(FAILED(D3DXCreateTextureFromFile(Device, "steelfloor.jpg", &FloorTex)))return 0;
	D3DXCreateTextureFromFile(Device, "brickwall.jpg", &WallTex);
	D3DXCreateTextureFromFile(Device, "ice.bmp", &MirrorTex);
	
	//Set up the view matrix
	D3DXVECTOR3 pos(0.0f, 5.0f, -5.0f);
	D3DXVECTOR3 target(0.0f, 0.0f,0.0f);
	D3DXVECTOR3 up(0.0f, 1.0f,0.0f);
	D3DXMatrixLookAtLH(&mView, &pos, &target, &up);
	
	//Set world matrix to idendity
	D3DXMatrixIdentity(&mWorld);
	//
	// Set the projection matrix.
	//	
	D3DXMatrixPerspectiveFovLH(
		&mproj,                        // result
		D3DX_PI *  0.125f,               // 90 - degrees
		(float)Width / (float)Height, // aspect ratio
		0.1f,                         // near plane
		1000.0f);                     // far plane  

	setUpRenderStates();
	ShowCursor(false);
	return true;
}

void Cleanup()
{
	d3d::Release<IDirect3DVertexBuffer9*>(VB);
	d3d::Release<IDirect3DTexture9*>(FloorTex);
	d3d::Release<IDirect3DTexture9*>(WallTex);
	d3d::Release<IDirect3DTexture9*>(MirrorTex);
	myMesh->Release();
}


void ClearBuffer(DWORD dwColor)
{
	Device->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER , 0xffffffff , 1.0f, 0L );
}

/////////////////////////////////////////////////////////////////////////////////////////
// MAIN LOOP
/////////////////////////////////////////////////////////////////////////////////////////
bool Display(float timeDelta)
{
	if( Device )
	{	
		// Update: Update the camera.
			
		if( ::GetAsyncKeyState('W') & 0x8000f )// Move forward
			TheCamera.walk(4.0f * timeDelta);

		if( ::GetAsyncKeyState('S') & 0x8000f )//Move backward
			TheCamera.walk(-4.0f * timeDelta);

		if( ::GetAsyncKeyState('A') & 0x8000f )//Strafe left
			TheCamera.strafe(-4.0f * timeDelta);

		if( ::GetAsyncKeyState('D') & 0x8000f )
			TheCamera.strafe(4.0f * timeDelta);//Strafe right

		if( ::GetAsyncKeyState('R') & 0x8000f )//Go up
			TheCamera.fly(4.0f * timeDelta);

		if( ::GetAsyncKeyState('F') & 0x8000f )//Go down
			TheCamera.fly(-4.0f * timeDelta);

		// Move Car
		if( ::GetAsyncKeyState('K')& 0x8000f )
		{
			CarPos.x+=0.1f; // move right
		}
		if( ::GetAsyncKeyState('L') & 0x8000f )
		{
			CarPos.x-=0.1f; // move left
		}
		if( ::GetAsyncKeyState('N') & 0x8000f )
		{
			CarPos.y+=0.1f; // move up
		}
		if( ::GetAsyncKeyState('M') & 0x8000f )
		{
			CarPos.y-=0.1f; // move down
			if (CarPos.y <= 1.5f) CarPos.y = 1.5f;
		}
		if( ::GetAsyncKeyState('C') & 0x8000f )
		{
			CarPos.x = 0;
			CarPos.y = 2.0f;
			CarPos.z = -15.0f; // Reset car position
		}
		if( ::GetAsyncKeyState('V') & 0x8000f )
		{
			D3DXVECTOR3 camPos(0.0f, 7.0f, -55.0f);
			TheCamera.setPosition(&camPos); // Reset Camera
		}
		
		//Place the camera
		TheCamera.getViewMatrix(&mView);
		Device->SetTransform(D3DTS_VIEW, &mView);

		rotY+=0.01f; // Model rotation

		// Read Mouse input if LEFT mouse button pressed
		if( ::GetAsyncKeyState(VK_LBUTTON))
			CameraMouseInput();
		
		// Reset cursor position
		int midScrX = GetSystemMetrics(SM_CXSCREEN) >> 1;
		int midScrY = GetSystemMetrics(SM_CYSCREEN) >> 1;
		SetCursorPos(midScrX, midScrY);
		
		// Render
		Device->Clear(0, 0, 
			D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL,
			0xff000000, 1.0f, 0L);
		Device->BeginScene();

		//Activate an Effect
		mFX->SetTechnique(mhTech);
		//any effect parameters that need to be updated must be set
		mFX->SetValue(mhvLightDirs, &(vLightDirs),sizeof(D3DXVECTOR4));
		mFX->SetValue(mhLightCol,&(lc) ,sizeof(D3DXCOLOR));
	
		RenderWorld();		
		drawShadow();
		RenderMirror();
		
		Device->EndScene();
		Device->Present(0,0,0,0); 
	}
	return true;
}


//
// WndProc
//
LRESULT CALLBACK d3d::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_DESTROY:
		::PostQuitMessage(0);
		break;

	case WM_KEYDOWN:
		if( wParam == VK_ESCAPE )
			::DestroyWindow(hwnd);
		break;

	}
	return ::DefWindowProc(hwnd, msg, wParam, lParam);
}

//
// WinMain
//
int WINAPI WinMain(HINSTANCE hinstance,
				   HINSTANCE prevInstance, 
				   PSTR cmdLine,
				   int showCmd)
{
	if(!d3d::InitD3D(hinstance,
		Width, Height, true, D3DDEVTYPE_HAL, &Device))
	{
		::MessageBox(0, "InitD3D() - FAILED", 0, 0);
		return 0;
	}

	if(!Setup())
	{
		::MessageBox(0, "Setup() - FAILED", 0, 0);
		return 0;
	}
	
	d3d::EnterMsgLoop( Display );

	Cleanup();

	Device->Release();

	return 0;
}